package com.github.jfcloud.gene.cache;

import cn.hutool.core.util.StrUtil;
import com.github.jfcloud.admin.api.sys.dto.UserDTO;
import com.github.jfcloud.admin.api.sys.dto.UserInfo;
import com.github.jfcloud.admin.api.sys.entity.SysUser;
import com.github.jfcloud.admin.api.sys.feign.RemoteUserService;
import com.github.jfcloud.admin.api.sys.vo.UserVO;
import com.github.jfcloud.common.core.util.R;
import feign.FeignException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Nullable;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 用户名id和昵称缓存，提高查询效率
 */
@Slf4j
@Component
public class UserIdNameCache {

    /**
     * 用户id和用户信息映射，没有签名图片
     */
    private final Map<Long, SysUser> idMap = new ConcurrentHashMap<>();

    /**
     * 用户昵称和用户id映射
     */
    private final Map<String, Long> nicknameMap = new ConcurrentHashMap<>();

    /**
     * 用户名和用户id映射
     */
    private final Map<String, Long> usernameMap = new ConcurrentHashMap<>();

    /**
     * 不存在的用户id
     */
    private final Set<Long> nonExistingUsers = new HashSet<>();

    @Autowired
    private RemoteUserService remoteUserService;

    /**
     * 每日5点整清空缓存
     */
    @Scheduled(cron = "0 0 5 * * ?")
    public void clearCache() {
        log.info("清空用户缓存");
        idMap.clear();
        nicknameMap.clear();
        usernameMap.clear();
        nonExistingUsers.clear();
    }

    /**
     * 获取用户列表缓存
     * 接口调用需要用token，无法在定时任务中调用
     */
    public void initCache() {
        log.info("刷新用户缓存");
        List<UserDTO> userList = remoteUserService.getUserList();
        userList.forEach(e -> {
            idMap.put(e.getUserId(), e);
            if (StrUtil.isNotBlank(e.getUsername())) {
                usernameMap.put(e.getUsername(), e.getUserId());
            }
            if (StrUtil.isNotBlank(e.getNickname())) {
                nicknameMap.put(e.getNickname(), e.getUserId());
            }
        });
    }

    /**
     * 根据用户id获取用户信息
     *
     * @param userId 用户id
     * @return 用户信息
     */
    @Nullable
    public SysUser getUserById(Long userId) {
        if (userId == null) {
            return null;
        }
        if (idMap.isEmpty()) {
            initCache();
        }
        if (nonExistingUsers.contains(userId)) {
            return null;
        }

        if (idMap.containsKey(userId)) {
            return idMap.get(userId);
        }

        try {
            R<UserInfo> result = remoteUserService.getUserById(userId);
            if (result.isOk() && Objects.nonNull(result.getData())) {
                SysUser sysUser = result.getData().getSysUser();
                idMap.put(userId, sysUser);
                if (StrUtil.isNotBlank(sysUser.getNickname())) {
                    nicknameMap.put(sysUser.getNickname(), userId);
                }
                if (StrUtil.isNotBlank(sysUser.getUsername())) {
                    usernameMap.put(sysUser.getUsername(), userId);
                }
                return sysUser;
            }
        } catch (FeignException e) {
            log.error("查询用户信息失败，userId={}，msg={}", userId, e.getMessage());
            nonExistingUsers.add(userId);
        }
        return null;
    }

    /**
     * 根据用户昵称获取用户id
     *
     * @param nickname 用户昵称
     * @return 用户id
     */
    @Nullable
    public Long getUserIdByNickname(String nickname) {
        if (StrUtil.isBlank(nickname)) {
            return null;
        }
        if (idMap.isEmpty()) {
            initCache();
        }
        return nicknameMap.get(nickname);
    }

    /**
     * 根据用户名获取用户id
     *
     * @param username 用户名
     * @return 用户id
     */
    @Nullable
    public Long getUserIdByUsername(String username) {
        if (StrUtil.isBlank(username)) {
            return null;
        }
        if (idMap.isEmpty()) {
            initCache();
        }
        return usernameMap.get(username);
    }

    /**
     * 根据用户id获取用户昵称
     *
     * @param userId 用户id
     * @return 昵称，如果不存在，返回空字符串
     */
    public String getNicknameByUserId(String userId) {
        return getNicknameByUserId(Long.parseLong(userId), "");
    }

    public String getNicknameByUserId(Long userId) {
        return getNicknameByUserId(userId, "");
    }

    public String getNicknameByUserId(Long userId, String defaultValue) {
        SysUser sysUser = getUserById(userId);
        String result = sysUser == null ? "" : sysUser.getNickname();
        return StrUtil.isBlank(result) ? defaultValue : result;
    }

    /**
     * 根据用户名获取用户昵称
     *
     * @param username 用户名
     * @return 用户昵称，不存在返回空字符串
     */
    public String username2Nickname(String username) {
        Long userId = getUserIdByUsername(username);
        return userId == null ? "" : getNicknameByUserId(userId);
    }

    /**
     * 根据用户id获取用户信息，包含签名图片和文字
     */
    @Nullable
    private UserVO getUserInfoByUserId(Long userId) {
        if (userId == null) {
            return null;
        }
        //检查用户id是否存在
        SysUser sysUser = getUserById(userId);
        if (sysUser == null) {
            return null;
        }

        try {
            return remoteUserService.getById(userId);
        } catch (FeignException e) {
            log.error("查询用户信息失败，userId={}，msg={}", userId, e.getMessage());
        }
        return null;
    }

    /**
     * 根据用户id获取电子签名图片路径
     */
    public String getSignPic(@Nullable Long userId) {
        log.info("获取用户 [userId={}] 电子签名图片路径", userId);
        return Optional.ofNullable(getUserInfoByUserId(userId))
                .map(UserVO::getEleSignaturePic)
                .orElse("");
    }

    /**
     * 根据用户id获取电子签名图片base64
     */
    public String getSignBase64(@Nullable Long userId) {
        return getSignBase64(userId, false);
    }

    /**
     * 根据用户id获取电子签名图片base64
     *
     * @param userId           用户id
     * @param withBase64Header 是否包含base64头 data:image/png;base64,
     */
    public String getSignBase64(@Nullable Long userId, boolean withBase64Header) {
        log.info("获取用户 [userId={}] 电子签名base64", userId);
        return Optional.ofNullable(getUserInfoByUserId(userId))
                .map(u -> {
                    String eleSignature = StrUtil.nullToEmpty(u.getEleSignature());
                    return withBase64Header ? eleSignature : eleSignature.substring(Math.max(0, eleSignature.indexOf(',') + 1));
                })
                .orElse("");
    }
}
