package com.github.jfcloud.gene.sample.service.biz;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.admin.api.sys.dto.message.MessageUserDTO;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import com.github.jfcloud.gene.common.constant.StrConstant;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.common.util.UserUtil;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.file.entity.FileInfo;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.file.vo.FileVo;
import com.github.jfcloud.gene.flow.entity.FlowAudit;
import com.github.jfcloud.gene.flow.service.FlowAuditService;
import com.github.jfcloud.gene.flow.service.NotifyService;
import com.github.jfcloud.gene.sample.entity.PathologicalPreExperiment;
import com.github.jfcloud.gene.sample.entity.SampleInfo;
import com.github.jfcloud.gene.sample.entity.SamplePathological;
import com.github.jfcloud.gene.sample.enums.SampleTypeEnum;
import com.github.jfcloud.gene.sample.mapper.SampleInfoMapper;
import com.github.jfcloud.gene.sample.mapper.SamplePathologicalMapper;
import com.github.jfcloud.gene.sample.service.PathologicalPreExperimentService;
import com.github.jfcloud.gene.sample.service.SampleEditService;
import com.github.jfcloud.gene.sample.service.SamplePathologicalService;
import com.github.jfcloud.gene.sample.vo.PreExperimentVo;
import com.github.jfcloud.gene.sample.vo.SamplePathologicalVo;
import com.github.jfcloud.gene.sample.vo.SampleSubmitVo;
import com.github.jfcloud.gene.sys.service.DBSystemPropertiesService;
import com.github.jfcloud.gene.util.WordDataService;
import com.github.jfcloud.rich.config.WordConfig;
import com.github.jfcloud.rich.vo.HtmlParseVo;
import com.github.jfcloud.rich.word.WordNodeFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class SamplePathologicalServiceImpl extends ServiceImpl<SamplePathologicalMapper, SamplePathological> implements SamplePathologicalService, SampleEditService {

    /**
     * 病理样本-样本信息附件类型
     */
    public static final String PATHOLOGICAL_SAMPLE_INFO = "pathological.sampleInfo";
    /**
     * 病理样本-抗体信息附件类型
     */
    public static final String PATHOLOGICAL_ANTIBODY_INFO = "pathological.antibodyInfo";

    private final SampleInfoMapper sampleInfoMapper;
    private final PathologicalPreExperimentService preExperimentService;
    private final FileInfoService fileInfoService;
    private final WordDataService wordDataService;
    private final UserIdNameCache userIdNameCache;
    private final FlowAuditService flowAuditService;
    private final DBSystemPropertiesService systemPropertiesService;
    private final NotifyService notifyService;

    @Override
    public void removeBySampleId(Long sampleId) {
        List<SamplePathological> pathologicalList = list(new LambdaQueryWrapper<>(SamplePathological.class)
                .select(SamplePathological::getId)
                .eq(SamplePathological::getSampleId, sampleId));
        if (pathologicalList.isEmpty()) {
            return;
        }

        Set<Long> idSet = pathologicalList.stream().map(SamplePathological::getId).collect(Collectors.toSet());
        removeBatchByIds(idSet);

        preExperimentService.remove(new LambdaQueryWrapper<>(PathologicalPreExperiment.class).in(PathologicalPreExperiment::getPathologicalId, idSet));
        fileInfoService.remove(new LambdaQueryWrapper<>(FileInfo.class).in(FileInfo::getRelateId, idSet));
    }

    @Override
    public void save(Long sampleId, SampleSubmitVo vo) {
        SamplePathologicalVo pathological = vo.getPathological();
        Assert.notNull(pathological, "病理样本送检不能为空");
        removeBySampleId(sampleId);

        //保存病理样本送检信息
        SamplePathological samplePathological = BeanUtil.copyProperties(pathological, SamplePathological.class);
        samplePathological.setId(CustomIdGenerator.nextId());
        samplePathological.setSampleId(sampleId);

        //保存预实验列表
        if (pathological.getPreExperiment()) {
            List<PathologicalPreExperiment> preExperiments = BeanUtil.copyToList(pathological.getExperiments(), PathologicalPreExperiment.class);
            preExperiments.forEach(item -> item.setPathologicalId(samplePathological.getId()));
            preExperimentService.saveBatch(preExperiments);
        } else {
            //清空预实验相关的参数
            samplePathological.setPreDate("")
                    .setPrePeople("")
                    .setPreSubjectName("")
                    .setPreApplicantId(null)
                    .setPreApplicantName("")
                    .setPreApplicantLeaderId(null)
                    .setPreApplicantLeaderName("")
                    .setPreContent("")
                    .setPreConclusion("");
        }
        samplePathological.insert();

        //保存附件列表
        List<FileInfo> fileInfos = new ArrayList<>();
        pathological.getSampleInfoFiles().forEach(f -> {
                    FileInfo fileInfo = new FileInfo()
                            .setRelateId(samplePathological.getId())
                            .setType(PATHOLOGICAL_SAMPLE_INFO)
                            .setName(f.getName())
                            .setUrl(f.getUrl());
                    fileInfos.add(fileInfo);
                });
        pathological.getAntibodyInfoFiles().forEach(f -> {
                    FileInfo fileInfo = new FileInfo()
                            .setRelateId(samplePathological.getId())
                            .setType(PATHOLOGICAL_ANTIBODY_INFO)
                            .setName(f.getName())
                            .setUrl(f.getUrl());
                    fileInfos.add(fileInfo);
                });
        if (CollUtil.isNotEmpty(fileInfos)) {
            fileInfoService.saveBatch(fileInfos);
        }

        log.info("保存病理样本送检信息成功");
        generate(sampleId);
    }

    @Override
    public void generate(Long sampleId) {
        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);
        SamplePathologicalVo pathologicalVo = getDetail(sampleId);

        //文件名称：课题名称-申请人-申请日期-随机四位.docx
        String applyDate = DateUtil.format(sampleInfo.getApplyTime(), DatePattern.CHINESE_DATE_PATTERN);
        String fileName = String.format("病理实验计划表-%s-%s-%s-%s.docx",
                StrUtil.nullToEmpty(pathologicalVo.getSubjectName()), StrUtil.nullToEmpty(sampleInfo.getApplicantName()),
                applyDate,
                RandomUtil.randomString(RandomUtil.BASE_CHAR_NUMBER_LOWER, 4));
        log.info("生成word 课题名称：{} 文件名称：{}", pathologicalVo.getSubjectName(), fileName);

        JSONObject dataMap = new JSONObject();
        wordDataService.fillDocCreator(dataMap);

        //富文本图片
        JSONArray images = new JSONArray();
        dataMap.put("images", images);

        //实验
        List<PreExperimentVo> experiments = pathologicalVo.getExperiments();
        if (pathologicalVo.getPreExperiment() && experiments.isEmpty()) {
            //默认添加一个实验
            experiments.add(new PreExperimentVo());
        }
        AtomicInteger experimentIndex = new AtomicInteger(0);
        experiments.forEach(item -> {
            item.setTitle("实验" + Convert.numberToChinese(experimentIndex.incrementAndGet(), false));
            if (StrUtil.isNotBlank(item.getFblx())) {
                item.setFblx(item.getFblx().replace("未封闭", "未封"));
            }
            if (StrUtil.isNotBlank(item.getXftj())) {
                item.setXftj(item.getXftj().replace("高温高压", "温压"));
            }
            if (StrUtil.isNotBlank(item.getSyjg())) {
                WordConfig wordConfig = new WordConfig().setMaxImageWidth(400);
                HtmlParseVo parseVo = WordNodeFactory.html2xml(item.getSyjg(), wordConfig);
                item.setSyjg(parseVo.getContent());
                images.addAll(parseVo.getImageCache());
            }
        });

        //预实验申请人负责人
        if (pathologicalVo.getPreApplicantLeaderId() != null) {
            String base64 = userIdNameCache.getSignBase64(pathologicalVo.getPreApplicantLeaderId());
            if (StrUtil.isNotBlank(base64)) {
                dataMap.put("preApplicantLeaderSign", base64);
            }
        }

        JSONObject sampleInfoObj = JSON.parseObject(JSON.toJSONStringWithDateFormat(sampleInfo, DatePattern.NORM_DATETIME_PATTERN));
        dataMap.put("sampleInfo", sampleInfoObj);
        JSONObject pathologicalObj = JSON.parseObject(JSON.toJSONStringWithDateFormat(pathologicalVo, DatePattern.NORM_DATETIME_PATTERN));
        dataMap.put("pathological", pathologicalObj);

        //申请时间
        String applyDateStr = DateUtil.format(sampleInfo.getApplyTime(), DatePattern.NORM_DATE_PATTERN);
        sampleInfoObj.put("applyDate", applyDateStr);

        //检测人员
        if (StrUtil.isNotEmpty(sampleInfo.getInspector())) {
            String inspectorName = JSON.parseArray(sampleInfo.getInspector())
                    .stream()
                    .map(user -> {
                        JSONObject obj = (JSONObject) user;
                        return obj.getString("name");
                    })
                    .collect(Collectors.joining("、"));
            sampleInfoObj.put("inspectorName", inspectorName);
        }

        //实验人员
        if (StrUtil.isNotEmpty(pathologicalVo.getPrePeople())) {
            String joinName = JSON.parseArray(pathologicalVo.getPrePeople())
                    .stream()
                    .map(item -> {
                        JSONObject obj = (JSONObject) item;
                        return obj.getString("nickname");
                    })
                    .collect(Collectors.joining("、"));
            pathologicalObj.put("prePeopleName", joinName);
        }

        //查询审核记录
        List<FlowAudit> flowAudits = flowAuditService.auditList(sampleId,"sample." + sampleInfo.getType(), true);
        //项目管理部审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.PROJECT_MANAGEMENT.getStatus().equals(audit.getFlowStatus()) && StrConstant.YES.equals(audit.getAuditResult()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("projectManagementDateStr", DateUtil.format(audit.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN));
                    String base64 = wordDataService.getBase64(audit.getCreateSign());
                    if (StrUtil.isNotBlank(base64)) {
                        dataMap.put("projectManagementSign", base64);
                    }
                    //审核内容
                    JSONObject auditObj = new JSONObject();
                    if (StrUtil.isNotBlank(audit.getAdditionalData())) {
                        auditObj = JSON.parseObject(audit.getAdditionalData());
                    }
                    auditObj.put("remarks", audit.getRemarks());
                    dataMap.put("projectManagementAudit", auditObj);
                });
        //部门负责人审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.DEPART_LEADER.getStatus().equals(audit.getFlowStatus()) && StrConstant.YES.equals(audit.getAuditResult()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("departLeaderDateStr", DateUtil.format(audit.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN));
                    String base64 = wordDataService.getBase64(audit.getCreateSign());
                    if (StrUtil.isNotBlank(base64)) {
                        dataMap.put("departLeaderSign", base64);
                    }
                    //审核内容
                    JSONObject auditObj = new JSONObject();
                    if (StrUtil.isNotBlank(audit.getAdditionalData())) {
                        auditObj = JSON.parseObject(audit.getAdditionalData());
                    }
                    auditObj.put("remarks", audit.getRemarks());
                    dataMap.put("departLeaderAudit", auditObj);
                });

        String modelName = "animalPathological.ftl";
        fileInfoService.uploadFileWithFileName(modelName, fileName, sampleInfo.getId(), sampleInfo.getStatus(), dataMap);
    }

    @Override
    public void afterExecute(Long sampleId) {
        //钉钉消息通知申请人、检测人员
        if (!systemPropertiesService.getBooleanValue("sample.afterExecute.dingding.enable")) {
            return;
        }

        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);

        String projectType = "样本送检/" + Objects.requireNonNull(SampleTypeEnum.resolve(sampleInfo.getType())).getLabel();
        if (StrUtil.isNotBlank(sampleInfo.getProjectNo())) {
            projectType += "/" + sampleInfo.getProjectNo();
        }
        if (StrUtil.isNotBlank(sampleInfo.getProjectName())) {
            projectType += "/" + sampleInfo.getProjectName();
        }
        String dingMsg = String.format("【%s】【%s】已执行项目", projectType, UserUtil.getNickName());

        Set<Long> userIds = new HashSet<>();
        //申请人
        userIds.add(sampleInfo.getApplicantId());
        //检测人员
        if (StrUtil.isNotBlank(sampleInfo.getInspector())) {
            JSON.parseArray(sampleInfo.getInspector()).forEach(item -> {
                JSONObject obj = (JSONObject) item;
                if (obj.containsKey("id")) {
                    userIds.add(obj.getLong("id"));
                }
            });
        }

        List<MessageUserDTO> users = userIds.stream()
                .map(x -> {
                    MessageUserDTO messageUserDTO = new MessageUserDTO();
                    messageUserDTO.setUserId(x);
                    return messageUserDTO;
                })
                .collect(Collectors.toList());
        notifyService.sendDingding("样本送检", dingMsg, users);
    }

    @Override
    public SamplePathologicalVo getDetail(Long sampleId) {
        SamplePathological samplePathological = getOne(new LambdaQueryWrapper<>(SamplePathological.class)
                .eq(SamplePathological::getSampleId, sampleId)
                .eq(SamplePathological::getDeleted, WhetherEnum.NO.getCode()));
        if (samplePathological == null) {
            return null;
        }

        SamplePathologicalVo pathologicalVo = BeanUtil.copyProperties(samplePathological, SamplePathologicalVo.class);

        //预实验列表
        List<PathologicalPreExperiment> preExperiments = preExperimentService.list(new LambdaQueryWrapper<>(PathologicalPreExperiment.class)
                .eq(PathologicalPreExperiment::getPathologicalId, samplePathological.getId()));
        pathologicalVo.getExperiments().addAll(BeanUtil.copyToList(preExperiments, PreExperimentVo.class));

        //附件列表
        List<FileInfo> fileInfos = fileInfoService.list(new LambdaQueryWrapper<>(FileInfo.class)
                .eq(FileInfo::getRelateId, samplePathological.getId())
                .eq(FileInfo::getDeleted, WhetherEnum.NO.getCode())
                .orderByAsc(FileInfo::getId));
        if (CollUtil.isNotEmpty(fileInfos)) {
            pathologicalVo.getSampleInfoFiles().addAll(fileInfos.stream()
                    .filter(item -> PATHOLOGICAL_SAMPLE_INFO.equals(item.getType()))
                    .map(item -> BeanUtil.copyProperties(item, FileVo.class))
                    .collect(Collectors.toList()));

            pathologicalVo.getAntibodyInfoFiles().addAll(fileInfos.stream()
                    .filter(item -> PATHOLOGICAL_ANTIBODY_INFO.equals(item.getType()))
                    .map(item -> BeanUtil.copyProperties(item, FileVo.class))
                    .collect(Collectors.toList()));
        }

        return pathologicalVo;
    }
}