package com.github.jfcloud.gene.sample.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.admin.api.sys.dto.message.MessageUserDTO;
import com.github.jfcloud.common.core.util.R;
import com.github.jfcloud.common.data.datascope.DataScope;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import com.github.jfcloud.gene.common.constant.StrConstant;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.common.util.UserUtil;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.file.vo.FileVo;
import com.github.jfcloud.gene.flow.entity.FlowAudit;
import com.github.jfcloud.gene.flow.service.FlowAuditService;
import com.github.jfcloud.gene.flow.service.FlowFileVersionService;
import com.github.jfcloud.gene.flow.service.NotifyService;
import com.github.jfcloud.gene.flow.vo.SynProjectVo;
import com.github.jfcloud.gene.sample.dto.SampleAuditDto;
import com.github.jfcloud.gene.sample.dto.SampleDetailDto;
import com.github.jfcloud.gene.sample.dto.SamplePageDto;
import com.github.jfcloud.gene.sample.entity.SampleInfo;
import com.github.jfcloud.gene.sample.enums.SampleTypeEnum;
import com.github.jfcloud.gene.sample.mapper.SampleInfoMapper;
import com.github.jfcloud.gene.sample.service.SampleEditService;
import com.github.jfcloud.gene.sample.service.SampleInfoService;
import com.github.jfcloud.gene.sample.service.biz.SampleAnimalImagingServiceImpl;
import com.github.jfcloud.gene.sample.service.biz.SampleAnimalServiceImpl;
import com.github.jfcloud.gene.sample.service.biz.SamplePathologicalServiceImpl;
import com.github.jfcloud.gene.sample.vo.SampleAuditVo;
import com.github.jfcloud.gene.sample.vo.SamplePageVo;
import com.github.jfcloud.gene.sample.vo.SampleSubmitVo;
import com.github.jfcloud.gene.sys.service.DBSystemPropertiesService;
import com.github.jfcloud.project.api.dto.LacmsProjectDTO;
import com.github.jfcloud.project.api.dto.ProjectFeignDTO;
import com.github.jfcloud.project.api.feign.RemoteProjectService;
import com.github.jfcloud.project.api.feign.SubProjectFeign;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;

@Slf4j
@Service
@RequiredArgsConstructor
public class SampleInfoServiceImpl extends ServiceImpl<SampleInfoMapper, SampleInfo> implements SampleInfoService {

    private final SampleAnimalServiceImpl sampleAnimalService;
    private final SamplePathologicalServiceImpl samplePathologicalService;
    private final SampleAnimalImagingServiceImpl sampleAnimalImagingService;
    private final FlowFileVersionService fileVersionService;
    private final UserIdNameCache userIdNameCache;
    private final FlowAuditService flowAuditService;
    private final NotifyService notifyService;
    private final DBSystemPropertiesService systemPropertiesService;
    private final RemoteProjectService remoteProjectService;
    private final SubProjectFeign subProjectFeign;

    /**
     * 根据样本检测类型获取对应的Service
     */
    private SampleEditService getServiceByType(String type) {
        SampleTypeEnum typeEnum = SampleTypeEnum.resolve(type);
        Assert.notNull(typeEnum, "样本检测类型参数错误 type=" + type);

        switch (Objects.requireNonNull(typeEnum)) {
            case PATHOLOGICAL:
                return samplePathologicalService;
            case ANIMAL:
                return sampleAnimalService;
            case ANIMAL_IMAGE:
                return sampleAnimalImagingService;
        }

        log.error("样本检测类型参数错误 type={}", type);
        throw new IllegalArgumentException("样本检测类型参数错误");
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveSample(SampleSubmitVo vo) {
        //检查项目编号的唯一性
        if (StrUtil.isNotBlank(vo.getProjectNo())) {
            long count = count(new LambdaQueryWrapper<>(SampleInfo.class)
                    .eq(SampleInfo::getProjectNo, vo.getProjectNo())
                    .eq(SampleInfo::getDeleted, WhetherEnum.NO.getCode()));
            Assert.isTrue(count < 1, "项目编号已存在");
        }

        SampleInfo sampleInfo = BeanUtil.copyProperties(vo, SampleInfo.class);
        sampleInfo.setInspector(vo.getInspector().toJSONString());
        sampleInfo.setId(CustomIdGenerator.nextId());
        sampleInfo.setStatus(GeneStatusEnum.DRAFT.getStatus());
        sampleInfo.setVersion(1);
        if (vo.getApplyTime() == null) {
            sampleInfo.setApplyTime(LocalDateTime.now());
        }
        sampleInfo.insert();

        getServiceByType(vo.getType()).save(sampleInfo.getId(), vo);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateSample(Long id, SampleSubmitVo vo) {
        SampleInfo sampleInfo = getById(id);
        Assert.notNull(sampleInfo, "样本检测信息不存在");

        //检查项目编号的唯一性
        if (StrUtil.isNotBlank(vo.getProjectNo())) {
            long count = count(new LambdaQueryWrapper<>(SampleInfo.class)
                    .ne(SampleInfo::getId, id)
                    .eq(SampleInfo::getProjectNo, vo.getProjectNo())
                    .eq(SampleInfo::getDeleted, WhetherEnum.NO.getCode()));
            Assert.isTrue(count < 1, "项目编号已存在");
        }

        BeanUtil.copyProperties(vo, sampleInfo);
        sampleInfo.setInspector(vo.getInspector().toJSONString());
        sampleInfo.updateById();

        getServiceByType(vo.getType()).save(id, vo);
    }

    @Override
    public void generate(Long id) {
        SampleInfo sampleInfo = getById(id);
        Assert.notNull(sampleInfo, "样本检测信息不存在");
        getServiceByType(sampleInfo.getType()).generate(id);
    }

    @Override
    public void submit(Long id) {
        SampleInfo sampleInfo = getById(id);
        Assert.notNull(sampleInfo, "样本检测信息不存在");
        Assert.isTrue(GeneStatusEnum.SUBMIT_STATUS.contains(sampleInfo.getStatus()), "流程状态错误");

        SampleInfo updateInfo = new SampleInfo().setId(id).setStatus(GeneStatusEnum.DEPART_LEADER.getStatus());
        //如果是驳回重新提交，修改申请日期和申请人
        if (GeneStatusEnum.REJECTED.getStatus().equals(sampleInfo.getStatus())) {
            log.info("流程 id={} 驳回后重新提交，更新申请日期和申请人", sampleInfo.getId());
            updateInfo.setVersion(sampleInfo.getVersion() + 1)
                    .setCreateBy(UserUtil.getUserId().toString())
                    .setCreateTime(new Date());
        }
        updateInfo.updateById();
        notify(id);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void copySample(Long id) {
        SampleDetailDto detailDto = getDetail(id);
        SampleSubmitVo vo = BeanUtil.copyProperties(detailDto, SampleSubmitVo.class);
        if (StrUtil.isNotEmpty(vo.getProjectName())) {
            vo.setProjectName(vo.getProjectName() + "(1)");
        }
        vo.setProjectNo("");
        vo.setApplyTime(null);
        saveSample(vo);
    }

    @Override
    public Page<SamplePageDto> getPage(SamplePageVo vo) {
        Page<SampleInfo> pageQuery = new Page<>(vo.getCurrent(), vo.getSize());
        LambdaQueryWrapper<SampleInfo> wrapper = new LambdaQueryWrapper<>(SampleInfo.class)
                .eq(StrUtil.isNotBlank(vo.getType()), SampleInfo::getType, vo.getType())
                .eq(StrUtil.isNotBlank(vo.getStatus()), SampleInfo::getStatus, vo.getStatus())
                .eq(Objects.nonNull(vo.getApplicantId()), SampleInfo::getApplicantId, vo.getApplicantId())
                .eq(SampleInfo::getDeleted, WhetherEnum.NO.getCode())
                .orderByDesc(SampleInfo::getCreateTime);
        if (StrUtil.isNotBlank(vo.getProject())) {
            wrapper.and(i -> i.like(SampleInfo::getProjectNo, vo.getProject()).or()
                    .like(SampleInfo::getProjectName, vo.getProject()));
        }
        Page<SampleInfo> page = baseMapper.selectPageByScope(pageQuery, wrapper, DataScope.of());

        Page<SamplePageDto> pageResult = new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
        if (page.getTotal() < 1) {
            return pageResult;
        }

        List<SamplePageDto> records = BeanUtil.copyToList(page.getRecords(), SamplePageDto.class);
        records.forEach(item -> {
            //查询下载链接
            FileVo latestFile = fileVersionService.getLatestFile(item.getId());
            item.setFileUrl(latestFile.getUrl());
            item.setFilename(latestFile.getName());
            item.setStatusLabel(Objects.requireNonNull(GeneStatusEnum.getByStatus(item.getStatus())).getDescription());
        });
        pageResult.setRecords(records);
        return pageResult;
    }

    @Override
    public SampleDetailDto getDetail(Long id) {
        SampleInfo sampleInfo = getById(id);
        Assert.notNull(sampleInfo, "样本检测信息不存在");

        SampleDetailDto detailDto = BeanUtil.copyProperties(sampleInfo, SampleDetailDto.class);
        if (StrUtil.isNotBlank(sampleInfo.getInspector())) {
            detailDto.setInspector(JSONArray.parseArray(sampleInfo.getInspector()));
        }

        SampleTypeEnum typeEnum = SampleTypeEnum.resolve(sampleInfo.getType());
        Assert.notNull(typeEnum, "未知样本类型 type=" + sampleInfo.getType());
        switch (Objects.requireNonNull(typeEnum)) {
            case PATHOLOGICAL:
                detailDto.setPathological(samplePathologicalService.getDetail(id));
                break;
            case ANIMAL:
                detailDto.setAnimal(sampleAnimalService.getDetail(id));
                break;
            case ANIMAL_IMAGE:
                detailDto.setAnimalImage(sampleAnimalImagingService.getDetail(id));
                break;
        }

        return detailDto;
    }

    @Override
    public void audit(Long id, SampleAuditVo vo) {
        SampleInfo sampleInfo = getById(id);
        Assert.notNull(sampleInfo, "样本检测信息不存在");
        Assert.isFalse(GeneStatusEnum.SUBMIT_STATUS.contains(sampleInfo.getStatus()), "样本检测流程状态错误");

        String nextStatus = "";
        GeneStatusEnum statusEnum = Objects.requireNonNull(GeneStatusEnum.getByStatus(sampleInfo.getStatus()));
        switch (statusEnum) {
            case DEPART_LEADER:
                Assert.isTrue(UserUtil.getUserId().equals(sampleInfo.getDepartLeaderId()), "抱歉，您不能审核本条记录");
                nextStatus = GeneStatusEnum.PROJECT_MANAGEMENT.getStatus();
                break;
            case PROJECT_MANAGEMENT:
                Assert.isTrue(UserUtil.getUserId().equals(sampleInfo.getProjectManageId()), "抱歉，您不能审核本条记录");
                nextStatus = GeneStatusEnum.COMPLETED.getStatus();
                break;
            default:
                log.info("流程项目 ({}) 状态为{}", sampleInfo.getProjectName(), statusEnum.getDescription());
                throw new IllegalArgumentException("流程状态错误");
        }

        //保存审批节点
        FlowAudit flowAudit = new FlowAudit()
                .setFlowId(id)
                .setFlowType("sample." + sampleInfo.getType())
                .setFlowStatus(sampleInfo.getStatus())
                .setAuditResult(vo.getAuditResult())
                .setRemarks(vo.getRemarks())
                .setCreateSign(userIdNameCache.getSignPic(UserUtil.getUserId()));
        if (StrUtil.isNotBlank(vo.getProcessMethod())) {
            JSONObject dataObj = new JSONObject().fluentPut("processMethod", vo.getProcessMethod());
            flowAudit.setAdditionalData(dataObj.toJSONString());
        }
        flowAudit.insert();

        //审核不通过
        if (StrConstant.NO.equals(vo.getAuditResult())) {
            log.info("流程项目 ({}) 审核不通过", sampleInfo.getProjectName());

            new SampleInfo()
                    .setId(id)
                    .setStatus(GeneStatusEnum.REJECTED.getStatus())
                    .updateById();

            generate(id);
            notify(id);
            return;
        }

        Assert.isTrue(StrConstant.YES.equals(vo.getAuditResult()), "审核结果错误");
        if (StrUtil.isNotBlank(vo.getApprovalNo())) {
            //检查批准编号的唯一性
            long count = count(new LambdaQueryWrapper<>(SampleInfo.class)
                    .eq(SampleInfo::getApprovalNo, vo.getApprovalNo())
                    .eq(SampleInfo::getDeleted, WhetherEnum.NO.getCode()));
            Assert.isTrue(count < 1, "项目批准编号已存在");
        }

        //审核通过
        log.info("流程项目 ({}) 审核通过", sampleInfo.getProjectName());

        sampleInfo = new SampleInfo();
        //更新项目批准编号、修订号等
        BeanUtil.copyProperties(vo, sampleInfo);
        sampleInfo.setId(id)
                .setStatus(nextStatus)
                .updateById();

        generate(id);
        notify(id);
    }

    @Override
    public List<SampleAuditDto> getAuditRecords(Long id) {
        SampleInfo sampleInfo = getById(id);
        Assert.notNull(sampleInfo, "样本检测信息不存在");

        List<FlowAudit> auditList = flowAuditService.auditList(id, "sample." + sampleInfo.getType(), false);
        if (auditList.isEmpty()) {
            return Collections.emptyList();
        }

        List<SampleAuditDto> auditDtos = new ArrayList<>();
        for (FlowAudit audit : auditList) {
            SampleAuditDto auditDto = BeanUtil.copyProperties(audit, SampleAuditDto.class);
            auditDto.setCreateName(userIdNameCache.getNicknameByUserId(audit.getCreateBy()));
            auditDto.setFlowStatusLabel(Objects.requireNonNull(GeneStatusEnum.getByStatus(audit.getFlowStatus())).getDescription());
            if (StrUtil.isNotBlank(audit.getAdditionalData())) {
                JSONObject jsonObject = JSONObject.parseObject(audit.getAdditionalData());
                auditDto.setProcessMethod(jsonObject.getString("processMethod"));
            }
            auditDtos.add(auditDto);
        }
        return auditDtos;
    }

    @Override
    public void execute(Long id, SynProjectVo vo) {
        if (!systemPropertiesService.getBooleanValue("sample.sync.ky")) {
            throw new RuntimeException("样本送检-科研同步配置未开启，不进行同步");
        }

        SampleInfo sampleInfo = getById(id);
        Assert.notNull(sampleInfo, "样本送检流程不存在");
        Assert.isTrue(GeneStatusEnum.COMPLETED.getStatus().equals(sampleInfo.getStatus()), "样本送检流程非完成状态");
        Assert.isNull(sampleInfo.getKyId(), "样本送检流程已执行");

        if (Objects.isNull(vo.getApprovalType())) {
            String approvalType = systemPropertiesService.getValue("sample." + sampleInfo.getType() + ".approvalType");
            Assert.notEmpty(approvalType, "项目类型为空，无法同步");
            vo.setApprovalType(Long.parseLong(approvalType));
        }

        //项目编号
        String projectNo = sampleInfo.getProjectNo();
        if (StrUtil.isBlank(projectNo)) {
            projectNo = sampleInfo.getApprovalNo();
        }
        Assert.notEmpty(projectNo, "项目编号为空，无法同步");

        //项目名称
        String projectName = sampleInfo.getProjectName();
        if (StrUtil.isBlank(projectName)) {
            projectName = Objects.requireNonNull(SampleTypeEnum.resolve(sampleInfo.getType())).getLabel() + "-" + projectNo;
        }

        //推送数据到科研项目，并得到科研项目id
        ProjectFeignDTO projectFeignDTO = new ProjectFeignDTO();
        projectFeignDTO.setName(projectName);
        projectFeignDTO.setLeader(vo.getProjectLeader());
        projectFeignDTO.setProjectType(vo.getApprovalType());
        projectFeignDTO.setProjectCode(projectNo);
        projectFeignDTO.setApplicantUserId(Long.parseLong(sampleInfo.getCreateBy()));

        //推送申请人nickname
        String leaderNickname = userIdNameCache.getNicknameByUserId(vo.getProjectLeader());
        projectFeignDTO.setLeaders(leaderNickname);

        log.info("样本送检推送科研项目：flowId={}, param={}", id, JSON.toJSONString(projectFeignDTO));
        R<Long> longR = remoteProjectService.initProject(projectFeignDTO);
        if (!longR.isOk()) {
            log.error("projectId={}推送科研失败，{}", id, longR);
        }
        Assert.isTrue(longR.isOk(), "操作失败，科研" + longR);
        Long kyId = longR.getData();
        new SampleInfo().setId(id).setKyId(kyId).updateById();

        if (kyId != null && systemPropertiesService.getBooleanValue("sample.sync.animal")) {
            //同步动物中心，生成动物中心项目信息
            LacmsProjectDTO lacmsProjectDTO = new LacmsProjectDTO();
            lacmsProjectDTO.setKyId(kyId);
            lacmsProjectDTO.setProjectName(projectName);
            lacmsProjectDTO.setProjectLeader(vo.getProjectLeader());
            lacmsProjectDTO.setPl(leaderNickname);
            lacmsProjectDTO.setEthicsCode(projectNo);
            lacmsProjectDTO.setProjectType("样本送检");
            lacmsProjectDTO.setProjectNum(projectNo);
            log.info("推送动物中心：flowId={}, param={}", id, JSON.toJSONString(lacmsProjectDTO));
            subProjectFeign.initProject(lacmsProjectDTO);
        } else {
            log.warn("样本送检-动物中心同步配置未开启，不进行同步");
        }

        log.info("样本送检项目 ({}) 同步成功", projectName);
        getServiceByType(sampleInfo.getType()).afterExecute(id);
    }

    private void notify(Long sampleId) {
        SampleInfo sampleInfo = getById(sampleId);
        if (sampleInfo == null) {
            return;
        }

        //查询流程前一个状态
        List<FlowAudit> flowAudits = flowAuditService.auditList(sampleId, "sample." + sampleInfo.getType(), false);
        FlowAudit previousAudit = null;
        String previousStatus = "";
        String previousUser = "";
        if (CollUtil.isNotEmpty(flowAudits)) {
            previousAudit = flowAudits.get(0);
            GeneStatusEnum statusEnum = GeneStatusEnum.getByStatus(previousAudit.getFlowStatus());
            previousStatus = statusEnum != null ? statusEnum.getDescription() : "";
            previousUser = userIdNameCache.getNicknameByUserId(previousAudit.getCreateBy());
        }

        String projectType = "样本送检/" + Objects.requireNonNull(SampleTypeEnum.resolve(sampleInfo.getType())).getLabel();
        String proNo = StrUtil.emptyToDefault(sampleInfo.getProjectNo(), sampleInfo.getApprovalNo());
        if (StrUtil.isNotBlank(proNo)) {
            projectType += "/" + proNo;
        }
        String projectName = sampleInfo.getProjectName();
        if (StrUtil.isBlank(projectName) && StrUtil.isNotBlank(proNo)) {
            projectName = Objects.requireNonNull(SampleTypeEnum.resolve(sampleInfo.getType())).getLabel() + "-" + proNo;
        }
        if (StrUtil.isNotBlank(projectName)) {
            projectType += "/" + projectName;
        }

        Long userId;
        String dingMsg;
        GeneStatusEnum statusEnum = Objects.requireNonNull(GeneStatusEnum.getByStatus(sampleInfo.getStatus()));
        switch (statusEnum) {
            case REJECTED:
                //驳回通知申请人
                userId = Long.valueOf(sampleInfo.getCreateBy());
                dingMsg = String.format("【%s】【%s/%s】不同意，请修正后再提交", projectType, previousStatus, previousUser);
                break;
            case DEPART_LEADER:
                userId = sampleInfo.getDepartLeaderId();
                dingMsg = String.format("【%s】【%s】已提交，请进行审查", projectType, userIdNameCache.getNicknameByUserId(sampleInfo.getCreateBy()));
                break;
            case PROJECT_MANAGEMENT:
                userId = sampleInfo.getProjectManageId();
                dingMsg = String.format("【%s】【%s/%s】已同意，请进行审查", projectType, previousStatus, previousUser);
                break;
            case COMPLETED:
                //执行
                userId = Long.valueOf(sampleInfo.getCreateBy());
                dingMsg = String.format("【%s】【%s/%s】已同意，请执行项目", projectType, previousStatus, previousUser);
                break;
            default:
                log.error("流程项目 ({}) 状态异常 {}", projectName, statusEnum.getDescription());
                return;
        }

        if (sampleInfo.getVersion() > 1) {
            dingMsg += "（第" + sampleInfo.getVersion() + "次申请）";
        }

        //查询是否需要钉钉通知
        String key = String.format("sample.%s.dingding.enable", sampleInfo.getStatus());
        if (systemPropertiesService.getBooleanValue(key)) {
            MessageUserDTO messageUserDTO = new MessageUserDTO();
            messageUserDTO.setUserId(userId);
            notifyService.sendDingding("样本送检", dingMsg, Collections.singletonList(messageUserDTO));
        }
    }
}


